@@ -23,7 +23,6 @@
 #define __GST_VIDEO_OVERLAY_H__
 
 #include <gst/gst.h>
-#include <gst/video/gstvideosink.h>
 
 G_BEGIN_DECLS
 
@@ -69,51 +68,32 @@ struct _GstVideoOverlayInterface {
   void (*set_window_handle)    (GstVideoOverlay *overlay, guintptr handle);
 };
 
-GST_VIDEO_API
 GType   gst_video_overlay_get_type (void);
 
 /* virtual function wrappers */
 
-GST_VIDEO_API
 gboolean        gst_video_overlay_set_render_rectangle  (GstVideoOverlay * overlay,
                                                          gint              x,
                                                          gint              y,
                                                          gint              width,
                                                          gint              height);
 
-GST_VIDEO_API
 void            gst_video_overlay_expose                (GstVideoOverlay * overlay);
 
-GST_VIDEO_API
 void            gst_video_overlay_handle_events         (GstVideoOverlay * overlay,
                                                          gboolean          handle_events);
 
-GST_VIDEO_API
 void            gst_video_overlay_set_window_handle     (GstVideoOverlay * overlay,
                                                          guintptr handle);
 
 /* public methods to dispatch bus messages */
-
-GST_VIDEO_API
 void            gst_video_overlay_got_window_handle     (GstVideoOverlay * overlay,
                                                          guintptr          handle);
 
-GST_VIDEO_API
 void            gst_video_overlay_prepare_window_handle (GstVideoOverlay * overlay);
 
-GST_VIDEO_API
 gboolean        gst_is_video_overlay_prepare_window_handle_message (GstMessage * msg);
 
-GST_VIDEO_API
-void            gst_video_overlay_install_properties    (GObjectClass    * oclass,
-                                                         gint              last_prop_id);
-
-GST_VIDEO_API
-gboolean        gst_video_overlay_set_property          (GObject         * object,
-                                                         gint              last_prop_id,
-                                                         guint             property_id,
-                                                         const GValue    * value);
-
 G_END_DECLS
 
 #endif /* __GST_VIDEO_OVERLAY_H__ */
